package unicrawler;

import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import unicrawler.crawler.CrawlerMonitor;
import unicrawler.crawler.CrawlerWorker;
import unicrawler.crawler.CrawlerWorkerCounter;
import unicrawler.downloader.WebpageDownloader;
import unicrawler.types.CrawlerURL;
import unicrawler.utils.PropertiesFile;

public class CrawlerEnvironment {
	
	private Logger logger = Logger.getLogger(CrawlerEnvironment.class.getName());
	
	private PropertiesFile properties;
	
	private ScheduledThreadPoolExecutor workers;
	private CrawlerWorkerCounter workCounter;
	private Thread monitor;
	
	private CrawlerURL mSeeds;
	private WebpageDownloader mDownloader;

	public void start(String propertyPath) {
		logger.info("Attempting to read property file");
		
		this.properties = new PropertiesFile();
		
		if ( ! this.properties.loadPropertyFile(propertyPath) ) {
			// Unable to load the properties file
			logger.error("Unable to load properties file with path " + propertyPath);
			return;
		}
		
		// Initialize the Scheduled thread-pool
		this.workers = new ScheduledThreadPoolExecutor(this.properties.getInt("thread.poolsize", 2));
		this.workers.setKeepAliveTime(this.properties.getInt("thread.keepalive", 30000), TimeUnit.MILLISECONDS);
		this.workers.allowCoreThreadTimeOut(true);
		
		// Initialize our (thread-safe) worker counter
		this.workCounter = new CrawlerWorkerCounter();
		
		// Initialize & Start monitoring our Scheduled Thread-pool
		this.monitor = new Thread(new CrawlerMonitor(this.workers, this.workCounter));
		this.monitor.setDaemon(true);
		this.monitor.start();
		
		// Setup the seed URL with a depth of 0
		this.mSeeds = new CrawlerURL(
				this.properties.get("crawler.seedurl", "http://users.cs.cf.ac.uk/David.W.Walker/head.html"),
				0
			);
		
		// Setup the downloader
		this.mDownloader = new WebpageDownloader(this);
		
		// Since this is the first seed URL - we'll just execute this worker instantly
		this.workers.execute(new CrawlerWorker(this, this.workers, this.workCounter, this.mSeeds));
		
//		this.workers.schedule(new CrawlerWorker(this.workers, this.workCounter),
//				this.properties.getInt("thread.pooldelay", 1000),
//				TimeUnit.MILLISECONDS
//			);
	}
	
	public PropertiesFile getProperties() {
		return this.properties;
	}
	
	public WebpageDownloader getDownloader() {
		return this.mDownloader;
	}
	
	public CrawlerURL getSeedUrls() {
		return this.mSeeds;
	}
	
	/*
	 * Main Application entrypoint
	 */
	public static void main(String args[]) {
		if (args.length < 1) {
			System.out.println("Usage: CrawlerEnvironment [path-to-crawler.properties]");
			return;
		}
		
		// configure the logger using our property file
		PropertyConfigurator.configure(args[0]);
		
		// Kick start the environment
		new CrawlerEnvironment().start(args[0]);
	}
}
